/*
	DIGGER.C
	REMOTE DIGITIZER/TRANSMITTER Program
 	DESCRIPTION: 80C88 "C" LANGUAGE MODULE FOR USE WITH DIGGER.ASM,
	JUNE 21, 1991
	COPYRIGHT CUTLER DIGITAL DESIGN 1991
	Rev.1  09-18-91 Reese T. Cutler << Original program >>
*/

#define version 1

/*	This program has been written to operate in a custom
	8088 computer controller has up to 3 analog inputs and a 
	serial I/O line.

	The program calls on various assembly language routines which can be
	found in the assembly language file DIGGER.ASM and which are described
	below.  In addition, some "C" language routines in this program
	can be called by assembly interrupt routines in DIGGER.ASM

	The DIGGER.ASM code has all the hooks necessary for the application
	to work in a standalone	romable manner.

*/


/*
	The following assembly language routines are available:

	INTON()			enables interrupts

	INTOFF()		disables interrupts

	SysInt(int state)	sets interrupt (on/off) according to
				the value in state.  If state is 0, interrupts
				are disabled, else enabled.  This routine
				in integer that indicates the state of
				interrupts, before change: 0 if they were
				disabled, 512 if they were enabled.

	ColdReset()		restarts the receiver board

	GetAChar()		returns a char in an integer

	PutAChar(int char)	sends a character, waits til done

	PutAWord(int word)	sends a word, waits til done

	LookAtChar()		checks to see if any chars in serial
				input buffer, if not, returns -1,
				if so, returns value for char
				does not remove char from buffer

	Pause(int tickcount)	waits for 'tickcount' ticks, each tick
				is 33.3 ms

	SetTick(int tickcount)	sets tick counter

long	GetTick()		returns tick counter value, tick counter
				counts down to zero, then stops until set
				again.

	Sleep()			powers down processor until next interrupt

	MEMPoke(int address, int data)
				loads a memory location (indicated by address)
				with a new value (indicated by data)

	MEMPeek(int address)	reads a memory location (indicated by address)

	IOPoke(int address, int data)
				loads an io location (indicated by address)
				with a new value (indicated by data)

	IOPeek(int address)	reads an io location (indicated by address)

	ResetAD()		resets a/d, takes 3 seconds

	ResetVCO()		resets VCO

	GetAD(int nextchan)	returns a/d value for current channel,
				after setting mux for next channel

*/


#include "\manxc\header\stdio.h"


/*******************************************************/
/* logical definitions                                 */
/*******************************************************/
#define	true	1
#define false	0
#define TRUE	1
#define	FALSE	0

/* Parallel port addresses */
#define	PARA	64
#define PARB	65
#define	PARC	66
#define	PARCON	67

/* default ticks per second */
#define	DefaultTickRate		300	/* hardware clock interrupt rate */
#define	DefaultTickLimit	3	/* divisor of hardware clock */
#define	DefaultTimeoutSeconds	20	/* Seconds of inactivity until revert*/
#define	DefaultStartChar	13	/* carriage return */
#define	DefaultStopChar		10	/* line feed */
#define	DefaultResyncRate	1440L	/* minutes between resync */
#define	DefaultBaudRate		9600	/* baud rate */

/* special characters */
#define BACKSPACE	8
#define	SPACE		32
#define CARRIAGERETURN	13
#define	LINEFEED	10
#define	BELL		7
#define	DELETE		127

/* commands */
#define	TIMEOUT	0
#define COMM1	1
#define COMM2	2
#define COMM3	3
#define COMM4	4
#define COMM5	5
#define COMM6	6
#define COMM7	7
#define COMM8	8
#define COMM9	9
#define COMM10	10
#define COMM11	11
#define COMM12	12
#define COMM13	13
#define COMM14	14
#define COMM15	15
#define COMM16	16
#define COMM17	17
#define COMM18	18
#define COMM19	19
#define COMM20	20

#define BADCOMM 21

/* nvram addresses */

#define	NVROffset	0x6000

#define	PasswordAdd	0
#define ValidNVRAdd	80
#define	StopCharAdd	160
#define	StartCharAdd	161
#define	ChanCountAdd	162
#define	FirstChanAdd	163
#define	SecondChanAdd	164
#define	ThirdChanAdd	165
#define	FourthChanAdd	166
#define	FifthChanAdd	167
#define	SixthChanAdd	168
#define	SeventhChanAdd	169
#define	EighthChanAdd	170
#define	TickLimitAdd	171
#define	TickRateAdd	173
#define	baudRateAdd	175
#define	TTCountAdd	177
#define	RSCountAdd	181
#define	NextAdd		185

#define	StartPassWord	"USGS"
#define	ValidMessage	"Digger, (c) Cutler Digital Design, 1991"


/* function predefinitions */
char upper(int c);
long GetTick();
long NVReadLong(int address);

/* global variables */
char	Password[80]=	{65,65,65,65,65,65,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			};
char	tPassword[80]=	{0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			,0,0,0,0,0,0,0,0,0,0
			};
int	pCount=6;
int	channelMap[8]=	{4,5,3,0,0,0,0,0};
int	channelCount=3;
int	NVFlag=FALSE;
int	tick=0,csec=0,second=0,minute=0,hour=0,day=0,year=0;
int	SampleCount=0,SampleDec=0;
int	data[8]={0,1,2,3,4,5,6,7};
int	TickLimit=DefaultTickLimit;
int	TickRate=DefaultTickRate;
long	TimeoutTickCount=DefaultTimeoutSeconds*DefaultTickRate;
int	StartChar=DefaultStartChar;
int	StopChar=DefaultStopChar;
long	resetCount=(DefaultResyncRate *(DefaultTickRate/DefaultTickLimit)*60);
long	resetCounter=(DefaultResyncRate *(DefaultTickRate/DefaultTickLimit)*60);
int	baudRate=DefaultBaudRate;

main()   /************************************************************/
{
char InputLine[80];
char	myChar;
int	index;

PrintString("\nU. S. Geological Survey Field Digitizer\n");
PrintString("(c) Cutler Digital Design 1991\n\n");

ResetVCO();
ResetAD();

/* Set up Password */
if(!RestoreFromNVR())
	SetDefault();

GetAD(channelMap[channelCount-1]);

ShowStatus();


/* Are there any input commands?  Return after 20 seconds of inactivity */
DoCommands();

SampleCount=0;
while (true)
	{
	if (SampleCount>0)
		{
		INTOFF();
		SampleDec+=1;
		INTON();
		resetCounter-=1;
		if (resetCounter == 0L)
			{
			ResetVCO();
			ResetAD();
			tick=0;
			SampleCount=0;
			SampleDec=0;
			resetCounter=resetCount;
			}
		for (index=0 ; index < channelCount ; index++)
			data[index]=GetAD(channelMap[index]);

		PutAChar(StartChar);

		for (index=0 ; index < channelCount ; index++)
			PutAWord(data[index]);

		PutAChar(StopChar);

		if (LookAtChar()!=-1)
			{
			CheckForPassword();
			}
		}
	else
		Sleep();
	}

} /*******  End main() ***********************************************/

/**********************************************************************
/
/      Title:	CheckForPassword()
/
***********************************************************************/
CheckForPassword()
{
int myChar;
char InputLine[80];
int index;

for ( index=0 ; index < pCount-1 ;index++)
	{
	tPassword[index]=tPassword[index+1];
	}
tPassword[pCount-1]=upper(GetAChar());

if(!strcmp(Password,tPassword))
	{
	for (index=0;index<=pCount;index++)
		tPassword[index]=0;

	/* Are there any input commands?  Return after 20 seconds of inactivity */
	DoCommands();
/*
	CR();
	PrintString("waiting for <cr> to continue ");
	GetLine(InputLine,80);
	PrintString(InputLine);
*/
	GetAD(channelMap[channelCount-1]);
	INTOFF();
	SampleCount=0;
	SampleDec=0;
	INTON();
	}
}

/**********************************************************************
/
/      Title:	DoCommands()
/
***********************************************************************/
DoCommands()
{

SetTick(TimeoutTickCount);
PrintString("\nEnter a command (type MENU for selection)\n");

while (GetTick()!=0)
	ExecuteCommand(GetCommand());
}


/**********************************************************************
/
/      Title:	PrintTime()
/
***********************************************************************/
/*
PrintTime()
{
char tm[80];
sprintf(tm,"%02d:%02d:%02d\n\r",hour,minute,second);
PrintString(tm);
}
*/

/**********************************************************************
/
/      Title:	GetLineT(char line[])
/
***********************************************************************/
GetLineT(char line[])
{
int gotLine;
int charcount;

line[0]=0;
gotLine=FALSE;
charcount=0;
SetTick(TimeoutTickCount);

while (!gotLine)
	{
	if (LookAtChar() == -1)
		Sleep();
	else
		{
		gotLine = ServeLine(line,80,&charcount);
		if (! gotLine) SetTick(TimeoutTickCount);
		}
	if (GetTick() == 0) gotLine = TRUE;
	} 


return( GetTick() > 0 );

}

/**********************************************************************
/
/      Title:	GetCommand()
/
***********************************************************************/
GetCommand()
{
int command;
char line[80];

if (GetLineT(line))
	{
	if (line[0]==0)
		command=BADCOMM;


	else if (!strcmp(line,"T"))
		command=COMM1;
	else if (!strcmp(line,"S"))
		command=COMM2;
	else if (!strcmp(line,"V"))
		command=COMM3;
	else if (!strcmp(line,"C"))
		command=COMM4;
	else if (!strcmp(line,"E"))
		command=COMM5;
	else if (!strcmp(line,"R"))
		command=COMM6;
	else if (!strcmp(line,"M"))
		command=COMM10;
	else if (!strcmp(line,"HELP"))
		command=COMM10;
	else if (!strcmp(line,"?"))
		command=COMM10;
	else if (!strcmp(line,"D"))
		command=COMM11;

	else if (!strcmp(line,"STATUS"))
		command=COMM1;
	else if (!strcmp(line,"SET"))
		command=COMM2;
	else if (!strcmp(line,"SAVE"))
		command=COMM3;
	else if (!strcmp(line,"COLLECT"))
		command=COMM4;
	else if (!strcmp(line,"RESET"))
		command=COMM5;
	else if (!strcmp(line,"RESTORE"))
		command=COMM6;
/*
	else if (!strcmp(line,"VALIDWRITE"))
		command=COMM7;
	else if (!strcmp(line,"LONGTEST"))
		command=COMM8;
	else if (!strcmp(line,"NVRAM"))
		command=COMM9;
*/
	else if (!strcmp(line,"MENU"))
		command=COMM10;
	else if (!strcmp(line,"DEFAULT"))
		command=COMM11;
	else if (!strcmp(line,"COMM12"))
		command=COMM12;
	else if (!strcmp(line,"COMM13"))
		command=COMM13;
	else if (!strcmp(line,"COMM14"))
		command=COMM14;

	else
		command=BADCOMM;
	}
else
	command=TIMEOUT;

return(command);
}

/**********************************************************************
/
/      Title:	ExecuteCommand(int command)
/
***********************************************************************/
ExecuteCommand(int command)
{
long templong;
char	line[80];
int temp;

if (command != TIMEOUT) 
	{
	templong=GetTick();
	SetTick(TimeoutTickCount);
	switch(command)
		{
		case COMM1:
			ShowStatus();
			break;
		case COMM2:
			DoSet();
			break;
		case COMM3:
			SaveToNVR();
			break;
		case COMM4:
			SetTick(0L);
			break;
		case COMM5:
			PrintString("\nResetting A/D and VCO \n");
			ResetVCO();
			ResetAD();
			SetTick(TimeoutTickCount);
			break;
		case COMM6:
			if (RestoreFromNVR())
				ShowStatus();
			else
				PrintString("\n\nCould not restore from NV ram\n");
			break;
/*
		case COMM7:
			PrintString("\nVALIDWRITEcommand\n");
			WriteValidMsg();
			break;
		case COMM8:
			sprintf(line,"\n %08ld value in nmicounter\n\n",templong);
			PrintString(line);
			SetTick(0x12345678L);
			PrintString("\n 12345678 sent value\n");
			templong=GetTick();
			sprintf(line," %08lx returned value\n\n",templong);
			PrintString(line);
			SetTick(TimeoutTickCount);
			break;
		case COMM9:
			DoNVRAM();
			break;
*/
		case COMM10:
			DoMenu();
			break;
		case COMM11:
			SetDefault();
			PrintString("\nDefault values set\n");
			break;
		case COMM12:
			PrintString("Command 12\n");
			break;
		case COMM13:
			PrintString("Command 13\n");
			break;
		case COMM14:
			PrintString("Command 14\n");
			break;

		case BADCOMM:
			PrintString("\nCommand not recognized, type MENU for list\n");
			break;
		default:
			break;
		}
	}
}

/**********************************************************************
/
/      Title:	DoMenu()
/
***********************************************************************/
DoMenu()
{
PrintString("\nCommands:\n");
PrintString("      STATUS - T - display current status\n");
PrintString("         SET - S - set various parameter values\n");
PrintString("        SAVE - V - save current parameter values permanently\n");
PrintString("     COLLECT - C - exit command mode\n");
PrintString("       RESET - E - reset a/d and vco\n");
PrintString("     RESTORE - R - restore parameter values from non volatile memory\n");
/*
PrintString("(V)ALIDWRITE - Write the validation message into NVRAM\n");
PrintString("    LONGTEST - test passing of long to asm routine\n");
*/
PrintString("        MENU - M - show commands\n");
PrintString("     DEFAULT - D - set all parameters to default values\n");
PrintString("\n\n");
}

/**********************************************************************
/
/      Title:	DoSet()
/
***********************************************************************/
DoSet()
{
int OpType;

OpType = 1;
SetTick(TimeoutTickCount);
while ( (OpType != 0 ) && (GetTick() > 0))
	{
	CR();
	PrintString("Set Routine:\n");
	PrintString("    1.  Indicate Hardware Clock Rate\n");
	PrintString("    2.  Indicate Hardware Baud Rate\n");
	PrintString("    3.  Set Sample Rate\n");
	PrintString("    4.  Set Inactivity Timeout\n");
	PrintString("    5.  Set Start Character\n");
	PrintString("    6.  Set Stop Character\n");
	PrintString("    7.  Set New Password\n");
	PrintString("    8.  Set Resync Interval\n");
	PrintString("    9.  Set Channels\n");
	CR();
	PrintString("    0.  Exit the Set Routine\n");
	CR();
	if (GetNumTD(&OpType, "Type the number of your selection:  "))
		{
		CR();
		CR();
		switch(OpType)
			{
			case 1:
				DoSetClock();
				break;
			case 2:
				DoSetBaudRate();
				break;
			case 3:
				DoSetFrequency();
				break;
			case 4:
				DoSetTimeout();
				break;
			case 5:
				DoSetStartChar();
				break;
			case 6:
				DoSetStopChar();
				break;
			case 7:
				DoSetPassword();
				break;
			case 8:
				DoSetResync();
				break;
			case 9:
				DoSetChannels();
				break;
			default:
				PrintString("\nBACK TO COMMAND MODE\n\n");
				break;
			}
		}
	else
		PrintString("\nTimeout during Set Routine\n");
	}
}

/**********************************************************************
/
/      Title:	DoNVRAM()
/
***********************************************************************/
/*
DoNVRAM()
{
int OpType;
char line[80];
int address,data;

OpType = 1;
SetTick(TimeoutTickCount);
PrintString("\nGot a NVRAM command\n");
while ( (OpType != 0 ) && (GetTick() > 0))
	{
	if (GetNumTD(&OpType, "type a number\n0 to quit, 1 to write, 2 to read,3 - next,4 -last\n"))
		{
		sprintf(line,"\nvalue back is %x\n",OpType);
		PrintString(line);
		switch(OpType)
			{
			case 1:
				if (GetNumTX(&address, "\nNVRAM write\ntype address in hex (0-1fff)\n"))
					if(GetNumTX(&data, "\ntype data word in hex "))
						NVWrite(address,data);
					else
						break;
				else
					break;
				data = data & 0xff;
				sprintf(line,"\n          %04x - %02x\n",address,data);
				PrintString(line);
				break;
			case 2:
				if (GetNumTX(&address, "\nNVRAM read\ntype address in hex (0-1fff)\n"))
					{
					data=NVRead(address);
					sprintf(line,"\n          %04x - %02x\n",address,data);
					PrintString(line);
					}
				else
					break;
				break;
			case 3:
				data=NVRead(++address);
				sprintf(line,"\n          %04x - %02x\n",address,data);
				PrintString(line);
				break;
			case 4:
				data=NVRead(--address);
				sprintf(line,"\n          %04x - %02x\n",address,data);
				PrintString(line);
				break;
			default:
				break;
			}
		}
	else
		PrintString("Timeout during type get in nvram ops\n");
	}
}
*/

/**********************************************************************
/
/      Title:	NVRead(int address)
/
***********************************************************************/
NVRead(int address)
{
return( MEMPeek( (address & 0x1fff) + NVROffset) );
}

/**********************************************************************
/
/      Title:	NVReadWord(int address)
/
***********************************************************************/
NVReadWord(int address)
{
unsigned temp;

temp=NVRead(address+1);
temp=temp <<8;
temp=temp | NVRead(address);
return( temp );
}

/**********************************************************************
/
/      Title:	long NVReadLong(int address)
/
***********************************************************************/
long NVReadLong(int address)
{
unsigned long temp;

temp=NVReadWord(address+2);

temp=(temp<<16) | (NVReadWord(address) & 0x0000ffff);

return(temp);
}

/**********************************************************************
/
/      Title:	NVWrite(int a1, int data)
/
***********************************************************************/
NVWrite(int a1, int data)
{
int limit;
int address;

data = data&0xff;
 
address=(a1 & 0x1fff)+ NVROffset;
limit=10000;

MEMPoke(address,data);
while ((limit-- >= 0) && (data != MEMPeek(address))) ;
}

/**********************************************************************
/
/      Title:	NVWriteWord(int a1, int data)
/
***********************************************************************/
NVWriteWord(int a1, int data)
{

NVWrite(a1,data);
NVWrite(a1+1,data>>8);
}

/**********************************************************************
/
/      Title:	NVWriteLong(int a1, long data)
/
***********************************************************************/
NVWriteLong(int a1, long data)
{
int temp;
long templ;

templ=data & 0x0000ffff;
temp=templ;
NVWriteWord(a1,temp);

templ=(data>>16) & 0x0000ffff;
temp=templ;
NVWriteWord(a1+2,temp);
}

/**********************************************************************
/
/      Title:	GetNumTD(val, msg)
/
***********************************************************************/
GetNumTD(val, msg)
int *val;
char msg[];
{
char line[80];

PrintString(msg);
if (GetLineT(line))
	{
	if(line[0]==0)
		*val=0;
	else
		if (sscanf(line,"%d",val) != 1 ) *val=0;
	return(true);
	}
else
	{
	return(FALSE);
	}
}

/**********************************************************************
/
/      Title:	GetNumTL(val, msg)
/
***********************************************************************/
GetNumTL(val, msg)
int *val;
char msg[];
{
char line[80];

PrintString(msg);
if (GetLineT(line))
	{
	if(line[0]==0)
		*val=0;
	else
		if (sscanf(line,"%ld",val) != 1) *val=0;
	return(true);
	}
else
	{
	return(FALSE);
	}
}

/**********************************************************************
/
/      Title:	GetNumTX(val, msg)
/
***********************************************************************/
GetNumTX(val, msg)
int *val;
char msg[];
{
char line[80];

PrintString(msg);
if (GetLineT(line))
	{
	if(line[0]==0)
		*val=0;
	else
		if (sscanf(line,"%x",val) != 1) *val=0;
	return(true);
	}
else
	{
	return(FALSE);
	}
}

/**********************************************************************
/
/      Title:	ServeLine(line,size,count)
/
***********************************************************************/
ServeLine(line,size,count)
char line[];
int size;
int *count;

{
char mychar;
int mychari;
int eol;

mychar=upper(GetAChar());
mychari=mychar;
if ( (mychari==CARRIAGERETURN) || (mychari == LINEFEED)
		|| (*count >= (size-1)))
	{
	line[*count]=0;
	eol=TRUE;
	}
else if ((mychari == BACKSPACE) || (mychari == DELETE) )
	{
	eol = FALSE;
	if (*count > 0)
		{
		(*count)--;
		PutAChar(BACKSPACE);
		PutAChar(SPACE);
		PutAChar(BACKSPACE);
		}
	else
		{
		PutAChar(BELL);
		}
	}
else
	{
	PutAChar(mychari);
	line[(*count)++]=mychar;
	eol = FALSE;
	}

return (eol);
}

/**********************************************************************
/
/      Title:	GetLine(line,size)
/
***********************************************************************/
/*
GetLine(line,size)
char line[];
int size;
{
int count;
char mychar;
int mychari;
int okay;

count=0;
okay=true;
do
	{
	mychar=GetAChar();
	mychari=mychar;
	if (mychari==CARRIAGERETURN)
		{
		line[count]=0;
		okay=false;
		}
	else if (mychari == LINEFEED)
		{
		line[count]=0;
		okay=false;
		}
	else if (count >= (size-1))
		{
		line[count]=0;
		okay=false;
		}		
	else if ((mychari == BACKSPACE) || (mychari == DELETE) )
		{
		if (count > 0)
			{
			count--;
			PutAChar(BACKSPACE);
			PutAChar(SPACE);
			PutAChar(BACKSPACE);
			}
		else
			{
			PutAChar(BELL);
			}
		}
	else
		{
		PutAChar(mychari);
		line[count++]=mychar;
		okay = true;
		}
	}
while (okay);
}
*/

/**********************************************************************
/
/      Title:	PrintString(ch)
/
***********************************************************************/
PrintString(ch)
char ch[];
{		
int count;
int mychar;

count = 0;
while(ch[count]!=0)
	{
	mychar=ch[count++];
	PutAChar(mychar);
	if (mychar=='\n') PutAChar('\r');
	}
}

/**********************************************************************
/
/      Title:	extern TickInt()
/
***********************************************************************/
extern TickInt()
{
int index;
int IntState;

tick+=1;

SampleCount-=SampleDec;
SampleDec=0;

if (tick>=TickLimit)
	{
	tick=0;
	SampleCount+=1;
	csec+=1;
	if (csec>99)
		{
		csec=0;
		second+=1;
		if (second >59)
			{
			second=0;
			minute+=1;
			if (minute>59)
				{
				minute=0;
				hour+=1;
				}
			}
		}
	}
}

/**********************************************************************
/
/      Title:	char upper(int c)
/
***********************************************************************/
char upper(int c)
{
if ( (c>='a') && (c<='z') )
	return(c-32);
else
	return(c);
}

/**********************************************************************
/
/      Title:	WriteValidMsg()
/
***********************************************************************/
WriteValidMsg()
{
char line[80];
char	c;
int count,nvrcount;

strcpy(line,ValidMessage);
count=0;
nvrcount=ValidNVRAdd;

while ( (c = line[count++]) != 0 )
	NVWrite(nvrcount++,c);
NVWrite(nvrcount,0);
}

/**********************************************************************
/
/      Title:	CheckNVRam()
/
***********************************************************************/
CheckNVRam()
{
char line[80];
char	c;
int count,nvrcount;

strcpy(line,ValidMessage);
count=0;
nvrcount=ValidNVRAdd;
NVFlag=FALSE;

while ( (c = line[count++]) != 0 )
	if ( ! (NVFlag = (NVRead(nvrcount++) == c) ) ) break;
return(NVFlag);
}

/**********************************************************************
/
/      Title:	ShowStatus()
/
***********************************************************************/
ShowStatus()
{
char line[80];

/* Header */
PrintString("\nStatus:\n\n");

/* Status of NV Ram */
ShowNVStatus();

/* Ticks per second */
ShowTickRate();

/* baud rate */
ShowBaudRate();

/* Ticks per sample period */
ShowTickLimit();

/* Show timeout */
ShowTimeout();

/* Show Start Character */
ShowStartChar();

/* Show Stop Character */
ShowStopChar();

/* Show the Reset Count */
ShowResetCount();

/* show all the channel selections */
ShowChannels();

CR();

/* Show the Password */
ShowPassword();

CR();
}

/**********************************************************************
/
/      Title:	ShowNVStatus()
/
***********************************************************************/
ShowNVStatus()
{
/* Status of NV Ram */
if (NVFlag)
	PrintString("    GOOD non volatile ram.\n");
else
	PrintString("     BAD non volatile ram.\n");
}

/**********************************************************************
/
/      Title:	ShowTickRate()
/
***********************************************************************/
ShowTickRate()
{
char line[80];

/* Ticks per second */
sprintf(line,"%8d Hardware interrupt frequency in HZ\n",TickRate);
PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowBaudRate()
/
***********************************************************************/
ShowBaudRate()
{
char line[80];

/* baud rate */
sprintf(line,"%8d Hardware baud rate\n",baudRate);
PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowTickLimit()
/
***********************************************************************/
ShowTickLimit()
{
char line[80];
float freq,freq1;
int temp1,temp2;

freq=TickRate;
freq=freq/TickLimit;
sprintf(line,"%8.2f Sample Frequency in HZ\n",freq);

PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowTimeout()
/
***********************************************************************/
ShowTimeout()
{
char line[80];
long temp;

temp=TimeoutTickCount/TickRate;
temp=temp/60;
/* Ticks per sample period */
sprintf(line,"%8ld Minutes of inactivity before TIMEOUT\n",temp);
PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowStartChar()
/
***********************************************************************/
ShowStartChar()
{
char line[80];

/* Start Character */
sprintf(line,"%8d START CHARACTER numeric value\n",StartChar);
PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowStopChar()
/
***********************************************************************/
ShowStopChar()
{
char line[80];

/* Stop Character */
sprintf(line,"%8d STOP CHARACTER numeric value\n",StopChar);
PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowResetCount()
/
***********************************************************************/
ShowResetCount()
{
char line[80];
long temp;

/* Reset count */
temp=resetCounter;
temp=temp*TickLimit;
temp=temp/60;
temp=temp/TickRate;
sprintf(line,"%8ld Minutes until next resync\n",temp);
PrintString(line);

temp=resetCount;
temp=temp*TickLimit;
temp=temp/60;
temp=temp/TickRate;
sprintf(line,"%8ld Minutes between resyncs\n",temp);
PrintString(line);

}

/**********************************************************************
/
/      Title:	ShowChannels()
/
***********************************************************************/
ShowChannels()
{
char line[80];
int i;

/* Channel count */
sprintf(line,"\n%8d Channels selected\n",channelCount);
PrintString(line);

PrintString("      channel  Sample Order\n");
sprintf(line,"%10d      sample(%d)\n",channelMap[channelCount-1],1);
PrintString(line);
for (i=0 ; i< channelCount-1; i++)
	{
	sprintf(line,"%10d      sample(%d)\n",channelMap[i],i+2);
	PrintString(line);
	}
}

/**********************************************************************
/
/      Title:	ShowPassword()
/
***********************************************************************/
ShowPassword()
{

/* Password */
PrintString("         Password is <");
PrintString(Password);
PrintString(">\n");

}

/**********************************************************************
/
/      Title:	CR()
/
***********************************************************************/
CR()
{

PrintString("\n");

}

/**********************************************************************
/
/      Title:	SaveToNVR()
/
***********************************************************************/
SaveToNVR()
{

PrintString("\nSaving parameters to NVRAM\n");
/* identifying message */
WriteValidMsg();

/*is NVRam good? */
CheckNVRam();

if (NVFlag)
	{
	/* save Password */
	WritePassword();

	NVWriteWord(TickRateAdd,TickRate);
	NVWriteWord(baudRateAdd,baudRate);
	NVWriteWord(TickLimitAdd,TickLimit);
	NVWriteLong(TTCountAdd,TimeoutTickCount);
	NVWriteLong(RSCountAdd,resetCount);
	NVWrite(StartCharAdd,StartChar);
	NVWrite(StopCharAdd,StopChar);
	NVWrite(ChanCountAdd,channelCount);
	NVWrite(FirstChanAdd,channelMap[0]);
	NVWrite(FirstChanAdd+1,channelMap[1]);
	NVWrite(FirstChanAdd+2,channelMap[2]);
	NVWrite(FirstChanAdd+3,channelMap[3]);
	NVWrite(FirstChanAdd+4,channelMap[4]);
	NVWrite(FirstChanAdd+5,channelMap[5]);
	NVWrite(FirstChanAdd+6,channelMap[6]);
	NVWrite(FirstChanAdd+7,channelMap[7]);

	PrintString("Write to NVRAM complete\n");
	}
else
	{
	PrintString("FAILURE to write to NVRAM\n");
	}

}

/**********************************************************************
/
/      Title:	RestoreFromNVR()
/
***********************************************************************/
RestoreFromNVR()
{

/*is NVRam good? */
CheckNVRam();

if (NVFlag)
	{
	ReadPassword();
	TickRate=NVReadWord(TickRateAdd);
	baudRate=NVReadWord(baudRateAdd);
	TickLimit=NVReadWord(TickLimitAdd);
	TimeoutTickCount=NVReadLong(TTCountAdd);
	resetCount=NVReadLong(RSCountAdd);
	resetCounter=resetCount;
	StartChar=NVRead(StartCharAdd);
	StopChar=NVRead(StopCharAdd);
	channelCount=NVRead(ChanCountAdd);
	channelMap[0]=NVRead(FirstChanAdd);
	channelMap[1]=NVRead(FirstChanAdd+1);
	channelMap[2]=NVRead(FirstChanAdd+2);
	channelMap[3]=NVRead(FirstChanAdd+3);
	channelMap[4]=NVRead(FirstChanAdd+4);
	channelMap[5]=NVRead(FirstChanAdd+5);
	channelMap[6]=NVRead(FirstChanAdd+6);
	channelMap[7]=NVRead(FirstChanAdd+7);
	return(TRUE);
	}
else
	{
	return(FALSE);
	}
}

/**********************************************************************
/
/      Title:	SetDefault()
/
***********************************************************************/
SetDefault()

{

DefaultPassword();

TickRate=DefaultTickRate;
baudRate=DefaultBaudRate;
TickLimit=DefaultTickLimit;
TimeoutTickCount=DefaultTimeoutSeconds;
TimeoutTickCount*=DefaultTickRate;
resetCount=(DefaultResyncRate *(DefaultTickRate/DefaultTickLimit)*60);
resetCounter=resetCount;
StartChar=DefaultStartChar;
StopChar=DefaultStopChar;

channelCount=3;
channelMap[0]=4;
channelMap[1]=5;
channelMap[2]=3;
channelMap[3]=0;
channelMap[4]=0;
channelMap[5]=0;
channelMap[6]=0;
channelMap[7]=0;
}

/**********************************************************************
/
/      Title:	DefaultPassword()
/
***********************************************************************/
DefaultPassword()
{
strcpy(Password,StartPassWord);
pCount=0;
while (Password[pCount])
	{
	Password[pCount]=upper(Password[pCount]);
	tPassword[pCount++]=0;
	}
tPassword[pCount]=0;
}

/**********************************************************************
/
/      Title:	ReadPassword()
/
***********************************************************************/
ReadPassword()
{
int	count;

count=0;
while ((Password[count]=NVRead(PasswordAdd+count)) != 0) count++;
pCount=0;
while (Password[pCount])
	{
	Password[pCount]=upper(Password[pCount]);
	tPassword[pCount++]=0;
	}
tPassword[pCount]=0;
}

/**********************************************************************
/
/      Title:	WritePassword()
/
***********************************************************************/
WritePassword()
{
char line[80];
char	c;
int count,nvrcount;

count=0;
nvrcount=PasswordAdd;

while ( (c = Password[count++]) != 0 )
	NVWrite(nvrcount++,c);
NVWrite(nvrcount,0);
}

/**********************************************************************
/
/      Title:	DoSetClock()
/
***********************************************************************/
DoSetClock()
{
int temp;
int oldTickRate;

PrintString("Note: To change the actual tick rate a jumper in hardware must be moved\n");
PrintString("      This parameter should be set to match the hardware setting\n\n");
oldTickRate=TickRate;
ShowTickRate();
if (GetNumTD(&temp, "\nType the hardware clock rate (1200,600,300): "))
	{
	switch(temp)
		{
		case 1:
		case 12:
		case 120:
		case 1200:
			TickRate=1200;
			break;
		case 3:
		case 30:
		case 300:
			TickRate=300;
			break;
		case 6:
		case 60:
		case 600:
			TickRate=600;
			break;
		default:
			break;
		}
/* adjust frequency, this should also adjust Resync,Inactivity and pcount */
	FreqAdjust(oldTickRate);
	CR();
	CR();
	ShowTickRate();
	}
}

/**********************************************************************
/
/      Title:	DoSetBaudRate()
/
***********************************************************************/
DoSetBaudRate()
{
int temp;

PrintString("Note: To change the actual baud rate a jumper in hardware must be moved\n");
PrintString("      This parameter should be set to match the hardware setting\n\n");
ShowBaudRate();
if (GetNumTD(&temp, "\nType the hardware baud rate (9600,4800,2400,1200,600,300): "))
	{
	switch(temp)
		{
		case 1:
		case 12:
		case 120:
		case 1200:
			baudRate=1200;
			break;
		case 2:
		case 24:
		case 240:
		case 2400:
			baudRate=2400;
			break;
		case 3:
		case 30:
		case 300:
			baudRate=300;
			break;
		case 4:
		case 48:
		case 480:
		case 4800:
			baudRate=4800;
			break;
		case 6:
		case 60:
		case 600:
			baudRate=600;
			break;
		case 9:
		case 96:
		case 960:
		case 9600:
			baudRate=9600;
			break;
		default:
			break;
		}	
	ChannelCountCheck();
	CR();
	CR();
	ShowBaudRate();
	}
}

/**********************************************************************
/
/      Title:	DoSetFrequency()
/
***********************************************************************/
DoSetFrequency()
{
int temp;
int oldTickLimit;

oldTickLimit=TickLimit;
ShowTickLimit();
if (GetNumTD(&temp, "\nType the desired sample rate (HZ): "))
	{
	if (temp != 0)
		{
		if (temp<1) temp=1;
		TickLimit=TickRate/temp;
		if (TickLimit<1) TickLimit=1;
		FreqChangeCleanup(TickRate,oldTickLimit);
		CR();
		CR();
		ShowTickLimit();
		}
	}
}

/**********************************************************************
/
/      Title:	DoSetTimeout()
/
***********************************************************************/
DoSetTimeout()
{
int temp;
long templong;
long templong1;

ShowTimeout();
if (GetNumTD(&temp, "\nType number of minutes of inactivity before timeout (1-60) : "))
	{
	if (temp != 0)
		{
		if (temp<1) temp=1;
		else if (temp>60) temp=60;
		templong=temp;
		templong1=TickRate;
		TimeoutTickCount=templong1*templong*60;
		CR();
		CR();
		ShowTimeout();
		}
	}
}

/**********************************************************************
/
/      Title:	DoSetStartChar()
/
***********************************************************************/
DoSetStartChar()
{
int temp;

ShowStartChar();
if (GetNumTD(&temp, "\nType numeric value of the start character (0-255): "))
	{
	if (temp<0) temp=0;
	else if (temp>255) temp=255;
	StartChar=temp;
	CR();
	CR();
	ShowStartChar();
	}
}

/**********************************************************************
/
/      Title:	DoSetStopChar()
/
***********************************************************************/
DoSetStopChar()
{
int temp;

ShowStopChar();
if (GetNumTD(&temp, "\nType numeric value of the start character (0-255): "))
	{
	if (temp<0) temp=0;
	else if (temp>255) temp=255;
	StopChar=temp;
	CR();
	CR();
	ShowStopChar();
	}
}

/**********************************************************************
/
/      Title:	DoSetPassword()
/
***********************************************************************/
DoSetPassword()
{
char line[80];

ShowPassword();
PrintString("\nType new password\n\n:");
if (GetLineT(line))
	{
	if (line[0] != 0)
		{
		strcpy(Password,line);
		pCount=0;
		while (Password[pCount])
			{
			Password[pCount]=upper(Password[pCount]);
			tPassword[pCount++]=0;
			}
		tPassword[pCount]=0;
		CR();
		CR();
		ShowPassword();
		}
	}
}

/**********************************************************************
/
/      Title:	DoSetResync()
/
***********************************************************************/
DoSetResync()
{
int temp;

ShowResetCount();
PrintString("\nType the length (in minutes) of the resync interval.\n");
PrintString("    (There are 1440 minutes in a day)");
if (GetNumTD(&temp, "\n\n( 10 - 10080 ) : "))
	{
	if (temp<10)  temp=10;
	else if (temp>10080) temp=10080;
	resetCount=temp;
	resetCount=(resetCount * 60); /* change to secs */
	resetCount=resetCount * TickRate;
	resetCount=resetCount / TickLimit;
	resetCounter=resetCount;
	CR();
	CR();
	ShowResetCount();
	}
}

/**********************************************************************
/
/      Title:	DoSetChannels()
/
***********************************************************************/
DoSetChannels()
{
char line[80];
int temp;
int chancnt,chan[8];
int getOut;
int maxCount;

maxCount=MaxChannelCount();
ShowChannels();


sprintf(line,"\n\n How many channels? (1-%d) :",maxCount);
		
if (GetNumTD(&chancnt,line))
	{
	if (chancnt != 0)
		{
		if (chancnt>maxCount) chancnt=maxCount;
		if (chancnt<1) chancnt=1;
		for (temp=0;temp<chancnt;temp++)
			{
			if(!GetNumTD(&chan[temp],"\nChannel : "))
				break;
			if (getOut = (chan[temp]==0))
				break;
			}
		if (GetTick()!=0)
			{
			if (!getOut)
				{
				channelCount=chancnt;
				for (temp=0 ; temp <chancnt-1; temp++)
					channelMap[temp]=chan[temp+1];
				channelMap[chancnt-1]=chan[0];
				}
			}
		}
	if (GetTick()!=0)
		{
		CR();
		CR();
		ShowChannels();
		}
	}
}

/**********************************************************************
/
/      Title:	MaxChannelCount()
/
***********************************************************************/
MaxChannelCount()
{
long templ;
int temp;

templ=baudRate;
templ=templ/10;
templ=templ*TickLimit;
templ=templ/TickRate;
templ=templ - 3 ; /* a free char, a start char and a stop char, 3 total */
templ=templ/2;
temp=templ;
if (temp<1) temp=1;
if (temp>8) temp=8;
return(temp);
}

/**********************************************************************
/
/      Title:	FreqAdjust(int oldRate)
/
***********************************************************************/
FreqAdjust(int oldRate)
{
int oldTickLimit;

oldTickLimit=TickLimit;

switch(oldRate)
	{
	case 1200:
		switch(TickRate)
			{
			case 1200:
				break;
			case 600:
				TimeoutTickCount=TimeoutTickCount/2;
				TickLimit=TickLimit/2;
				if (TickLimit<1) TickLimit=1;
				break;
			case 300:
				TimeoutTickCount=TimeoutTickCount/4;
				TickLimit=TickLimit/4;
				if (TickLimit<1) TickLimit=1;
				break;
			}
		break;
	case 600:
		switch(TickRate)
			{
			case 1200:
				TimeoutTickCount=TimeoutTickCount*2;
				TickLimit=TickLimit*2;
				break;
			case 600:
				break;
			case 300:
				TimeoutTickCount=TimeoutTickCount/2;
				TickLimit=TickLimit/2;
				if (TickLimit<1) TickLimit=1;
				break;
			}
		break;
	case 300:
		switch(TickRate)
			{
			case 1200:
				TimeoutTickCount=TimeoutTickCount*4;
				TickLimit=TickLimit*4;
				break;
			case 600:
				TimeoutTickCount=TimeoutTickCount*2;
				TickLimit=TickLimit*2;
				break;
			case 300:
				break;
			}
		break;
	}


FreqChangeCleanup(oldRate,oldTickLimit);
}

/**********************************************************************
/
/      Title:	FreqChangeCleanup(int oldRate, int oldLimit)
/
***********************************************************************/
FreqChangeCleanup(int oldRate, int oldLimit)
{
char line[80];

resetCount=resetCount / TickLimit;
resetCount=resetCount * oldLimit ;
resetCount=resetCount / oldRate  ;
resetCount=resetCount * TickRate ;
resetCounter=resetCount;

ChannelCountCheck();
}

/**********************************************************************
/
/      Title:	ChannelCountCheck()
/
***********************************************************************/
ChannelCountCheck()
{
int MCC;

MCC=MaxChannelCount();
if(channelCount>MCC)
	{
	channelMap[MCC-1]=channelMap[channelCount-1];
	channelCount=MCC;
	}	
}

/**********************************************************************
/
/      Title:	
/
***********************************************************************/
