/* FILE: mdigirec.c                                 (R. Cutler 04/30/91)
                                              (D. Tottingham  09/29/90)

This is a collection of C functions that manage the DIGIREC board for xdetect.
All functions have been written and compiled medium model.  The following
functions are included:

dr_buffer_full ()               return buffer status
dr_dump_configuration ()        dump the DT2821 configuration
dr_get_collect_info ()          return DM_INFO structure
dr_get_buffer_size ()           get buffer size
dr_get_channel_size ()          get channel block size
dr_get_channel_gain ()          get channel gain
dr_get_clock_source ()          get clock source
dr_get_digitization_rate ()     get digitization rate
dr_get_new_buffers ()           get new buffer(s) from DT2821
dr_get_next_buffer ()           get next buffer from mux queue
dr_get_number_of_ext_buffers () get number of external buffers
dr_get_scan_count ()            get scan count
dr_get_trigger_source ()        get trigger source
dr_initialize_buffers ()        initialize buffers
dr_initialize_collection ()     initialize the digirec
dr_initialize_params()          initialize parameters
dr_set_channel ()               put channel and gain in channel and gain lists
dr_set_ChannelBlocksize ()      set channel block size
dr_set_ChannelGain ()           set the channel gain
dr_set_ClockSource ()           set clock source
dr_set_DigitizationRate ()      set the digitization rate
dr_set_TriggerSource ()         set trigger source
dr_start_collection ()          start continuous data acquisition
dr_stop_collection ()           stop continuous data acquisition

EXTERNAL FUNCTIONS CALLED:

er_abort ()                     display an error message then quit
o_write_logfile ()              write string to log file stream
suds_initialize ()              initialize a suds structure
suds_initialize_tag ()          initialize a suds structtag

HISTORY:

V2.001   (04/30/91)	changed to allow other data acquisition systems, the
			information data structure was renamed from DT_28XX
			to DM_INFO, and move to mdemux.h
			(Reese T. Cutler)


*/



/*************************************************************************
                            INCLUDE FILES

*************************************************************************/
#include <conio.h>
#include <malloc.h>
#include <math.h>
#include <stdio.h>
#include <dos.h>

#include "mconst.h"
#include "mdemux.h"
#include "mdigirec.h"
#include "mdigisup.h"
#include "merror.h"
#include "mfile.h"
#include "mlog.h"
#include "mqueue.h"
#include "msudsini.h"
#include "mstation.h"
#include "mutils.h"
#include "xdetect.h"


/*************************************************************************
                                GLOBALS

*************************************************************************/
PRIVATE DM_INFO adc_info, far * a = &adc_info;

PRIVATE DR_CONFIGURATION config, far * c = &config;

PRIVATE int status;
PRIVATE Q_QUEUE mux_queue;
PRIVATE Q_LINK * next_ptr;
PRIVATE double digitization_rate;
PRIVATE unsigned long buffer_size;
PRIVATE unsigned long channel_blocksize;
PRIVATE unsigned int scan_count;
PRIVATE unsigned int channel_gain;

/*=======================================================================*
 *                         allocate_int_buffers                          *
 *=======================================================================*/
/* Allocate "internal" buffer queue for digirec.  All internal buffers
   reside in the base 640K of user RAM.                                  */

PRIVATE
void allocate_int_buffers ()
{
   static unsigned int far int_buffer_number;
   unsigned int actual_blocksize, actual_buffersize;
   double seconds_in_buffer;
   unsigned long offset;
   Q_TYPE type;
   Q_BUFFER far * mux;
   char * netwname;

   actual_buffersize = (buffer_size <= MAX_BUFFER_SIZE) ? buffer_size : MAX_BUFFER_SIZE;
   actual_blocksize = actual_buffersize / scan_count;
   seconds_in_buffer = actual_blocksize / digitization_rate;

   q_initialize (&mux_queue);
   netwname = st_get_netwname ();

   for (offset = 0; offset < buffer_size; offset += MAX_BUFFER_SIZE) {
      mux = (Q_BUFFER far *) _fmalloc (sizeof (Q_BUFFER));
      if (mux == NULL) er_abort (DR_INT_MEM);

      mux->data = (int far *) _fmalloc ( actual_buffersize * sizeof(unsigned));
      if (mux->data == NULL) er_abort (DR_INT_MEM);

      mux->buffer_size = actual_buffersize;
      mux->seconds_in_buffer = seconds_in_buffer;

      suds_initialize (MUXDATA, &mux->info);
      suds_initialize_tag (MUXDATA, &mux->structtag);

      mux->structtag.len_data = actual_buffersize * sizeof(unsigned);
      u_strncpy (mux->info.netname, ((char far *)netwname), 4);
      mux->info.blocksize = actual_blocksize;
      mux->info.numchans = scan_count;
      mux->info.dig_rate = digitization_rate;
      mux->info.typedata = 's';
      type.buffer = mux;
      q_enqueue (&mux_queue, type);
   }
}

/*=======================================================================*
 *                         allocate_ext_buffers                          *
 *=======================================================================*/
/* Allocate an "external" buffers for digirec.  All external buffers reside
   ABOVE the base 1000K of user RAM.                                     */

PRIVATE
void allocate_ext_buffers ()
{
   static unsigned far ext_buffer_number;
int xbcnt;

dr_init_xm(1024);
xbcnt=dr_init_xm_buff(buffer_size);
a->info.extended_bufs = dr_128K_blocks_used();
if (Debug_enabled) printf("\nexternal buffer count is %d\n",xbcnt);
if (Debug_enabled) printf("#of 128K external blocks used %d\n",
	a->info.extended_bufs);
}

/*=======================================================================*
 *                           dr_buffer_full                              *
 *=======================================================================*/
/* Return buffer status.                                                 */

PUBLIC
FLAG dr_buffer_full ()
{
   static unsigned int far buffer_number, far count_remaining;
   return (dr_full_status());
}

/*=======================================================================*
 *                        dr_dump_configuration                          *
 *=======================================================================*/
/* Dump the DT2821 configuration to a stream.                            */

PUBLIC
void dr_dump_configuration (stream)
FILE * stream;
{
int i;

if (c->device_id != DIGIREC)
	fprintf(stream, "No device defined.\r\n");
else
	{
	fprintf (stream, "Device id is ");
	fprintf (stream, "DIGIREC");

	fprintf(stream, ", ");

	fprintf(stream, "single channel DMA");
	fprintf(stream, ".");

	fprintf(stream, "\r\n");
	fprintf(stream, "Base I/O address is %4x hex.\r\n",c->base_address);
	fprintf(stream, "%d Digital channels.\r\n",c->channel_count);
	}
}

/*=======================================================================*
 *                          dr_get_buffer_size                           *
 *=======================================================================*/
/* Get buffer size.                                                      */

PUBLIC
unsigned long dr_get_buffer_size ()
{
   return (buffer_size);
}

/*=======================================================================*
 *                         dr_get_channel_gain                           *
 *=======================================================================*/
/* Get channel gain.                                                     */

PUBLIC
unsigned int dr_get_channel_gain (channel)
unsigned int channel;
{
   return ((channel < c->channel_count) ? a->gain_list[channel] : channel_gain);
}

/*=======================================================================*
 *                         dr_get_channel_size                           *
 *=======================================================================*/
/* Get channel block size.                                               */

PUBLIC
unsigned long dr_get_channel_size ()
{
   return (channel_blocksize);
}

/*=======================================================================*
 *                           dr_get_clock_source                         *
 *=======================================================================*/
/* Get clock source.                                                     */

PUBLIC
char dr_get_clock_source ()
{
   return (a->info.timing_source);
}

/*=======================================================================*
 *                           dr_get_collect_info
 *=======================================================================*/
/* Return DM_INFO structure.                                             */

PUBLIC
DM_INFO far * dr_get_collect_info ()
{
   return (a);
}

/*=======================================================================*
 *                        dr_get_digitization_rate                       *
 *=======================================================================*/
/* Get digitization rate.                                                */

PUBLIC
double dr_get_digitization_rate ()
{
   return (digitization_rate);
}

/*=======================================================================*
 *                          dr_get_new_buffers                           *
 *=======================================================================*/
/* Wait for buffer to complete.  Copy external buffer into user
   accessible mux queue.                                                 */

PUBLIC
void dr_get_new_buffers ()
{
   unsigned long offset, longxmem;
   Q_LINK * head;

   /* Wait for a buffer to complete */

   offset = 0;
   longxmem=dr_get_dma_buffer();
   if ( longxmem != 0 )

      {
      head = next_ptr = mux_queue.head;
      while (head != NULL) {
	getxmem(longxmem+offset,fpta(head->type.buffer->data),
		(int) head->type.buffer->buffer_size);
        offset += head->type.buffer->buffer_size;
         head = head->next;
      }
   }
   else er_abort (DR_DMA_BLOCKED);

}

/*=======================================================================*
 *                          dr_get_next_buffer                           *
 *=======================================================================*/
/* Get next buffer from mux queue.                                       */

PUBLIC
Q_BUFFER far * dr_get_next_buffer ()
{
   Q_LINK * this_ptr;

   this_ptr = next_ptr;
   if (next_ptr != NULL)
      next_ptr = next_ptr->next;

   return (this_ptr->type.buffer);
}

/*=======================================================================*
 *                      dr_get_number_of_ext_buffers                     *
 *=======================================================================*/
/* Get number of external buffers.                                       */

PUBLIC
int dr_get_number_of_ext_buffers ()
{
   return (a->info.extended_bufs);
}

/*=======================================================================*
 *                            dr_get_scan_count                          *
 *=======================================================================*/
/* Get scan count.                                                       */

PUBLIC
int dr_get_scan_count ()
{
   return (scan_count);
}

/*=======================================================================*
 *                           dr_get_trigger_source                       *
 *=======================================================================*/
/* Get trigger source.                                                   */

PUBLIC
char dr_get_trigger_source ()
{
   return (a->info.trigger_source);
}

/*=======================================================================*
 *                           dr_initialize_buffers                       *
 *=======================================================================*/
/* Initialize buffers.                                                   */

PUBLIC
void dr_initialize_buffers ()
{
if (Debug_enabled) printf("entered dr initialize buffers\n");

/* scan_count is the number of stations producing 16 bit data */
/* channel_blocksize is the number of scans in a dma buffer, it must
	be a power of 2 */
/* buffer_size is the number of 16 bit words in each dma buffer */

   buffer_size = channel_blocksize * scan_count;
   buffer_size = (! buffer_size) ? SEGMENT_SIZE : buffer_size;
   channel_blocksize = buffer_size / scan_count;


   /* Do some error checking first */

   if (! u_ispow2( channel_blocksize))
      er_abort (DR_CBS_NP2);
   else if (buffer_size > SEGMENT_SIZE)
      er_abort (DR_BS_TOO_BIG);

   allocate_int_buffers ();

   allocate_ext_buffers ();

}


/*=======================================================================*
 *                           dr_initialize_collection
 *=======================================================================*/
/* Initialize the DIGIREC                    */

PUBLIC
void dr_initialize_collection ()
{
   unsigned int actual_scancount;
   int errnum;
   int i,j,k;

if (Debug_enabled) printf("got to DR_INITIALIZE_COLLECTION\n");
   /* Do some error checking */
if (Debug_enabled) printf("scan_count is %d\n",scan_count);
   if (!scan_count)
      er_abort (DR_SCAN_COUNT_0);
   else if (! u_ispow2( (unsigned long)scan_count))
      er_abort (DR_SCAN_COUNT_NP2);

   actual_scancount = (scan_count > c->channel_count) ? c->channel_count :
                      scan_count;

dr_set_io_add(c->base_address);

dr_reset();

k=0;
for (i=0; i<16 ; i++)
	{
	for(j=0;j<3; j++)
		{
		dr_set_position(i,j,ScanCheck(k));
		k+=1;
		}
	dr_set_position(i,3,ScanCheck(48+i));
	}
if (Debug_enabled) printf("Base I/O address is %4x hex.\r\n",c->base_address);

/**debug**/
if (Debug_enabled)
   printf ("Channel digitization rate = %10.4lf\n", digitization_rate);

}

/*=======================================================================*
 *                          dr_initialize_params                         *
 *=======================================================================*/
/* Initialize parameters.                                                */

PUBLIC
void dr_initialize_params()
{

   /* Get the current configuration */
	c->base_address=DR_BASE_ADDRESS;
		dr_set_io_add(c->base_address);
	c->dma_channel=DR_DMA_CHANNEL;
		dr_set_dma_chan(c->dma_channel);
	c->int_channel=DR_INT_CHANNEL;
		dr_set_int_chan(c->int_channel);
	c->device_id = DIGIREC;
	c->scan_count = 0;
	c->channel_count=64;

   digitization_rate = DIGITIZATION_RATE;
   channel_blocksize = 0;
   channel_gain = CHANNEL_GAIN;
   scan_count = 0;

   suds_initialize (ATODINFO, &a->info);
   suds_initialize_tag (ATODINFO, &a->structtag);

      a->info.base_address = c->base_address;
      a->info.device_id = c->device_id;
      a->info.device_flags = 0;

   a->info.trigger_source = 'i';
   a->info.timing_source = 'i';
   a->info.external_mux = 1;

   a->channel_list = (int far *) _fmalloc (c->channel_count * sizeof(int));
   if (a->channel_list == NULL) er_abort (DR_INT_MEM);

   a->gain_list = (int far *) _fmalloc (c->channel_count * sizeof(int));
   if (a->gain_list == NULL) er_abort (DR_INT_MEM);

}

/*=======================================================================*
 *                              dr_set_channel                           *
 *=======================================================================*/
/* Put channel number in channel list and gain number in gain list.      */

PUBLIC
void dr_set_channel (channel, gain)
unsigned int channel;
unsigned int gain;
{
static FLAG first_time = TRUE;
unsigned int i, max_channel;

if (channel != scan_count) er_abort (DR_BAD_CHAN);

if (scan_count < c->channel_count)
	{
	a->channel_list[scan_count] = channel;
	a->gain_list[scan_count] = (gain == 0) ? channel_gain : gain;
	if (a->gain_list[scan_count] > MAX_GAIN ||
		!u_ispow2((unsigned long) a->gain_list[scan_count]) )
		{
		er_abort (DR_BAD_GAIN);
		}
	}
else
	{
	er_abort(DR_SCAN_COUNT_BIG);
	}
   scan_count++;
}

/*=======================================================================*
 *                         dr_set_ChannelBlocksize                       *
 *=======================================================================*/
/* Set the channel block size.                                           */

PUBLIC
void dr_set_ChannelBlocksize (blocksize)
unsigned long blocksize;
{
   channel_blocksize = blocksize;
}

/*=======================================================================*
 *                            dr_set_ChannelGain                         *
 *=======================================================================*/
/* Set the channel gain.                                                 */

PUBLIC
void dr_set_ChannelGain (gain)
unsigned int gain;
{
   int i;

   if (gain > MAX_GAIN || !u_ispow2((unsigned long) gain))
	er_abort (DR_BAD_GAIN);
   channel_gain = gain;
   for (i = 0; i < scan_count && i < c->channel_count; i++)
      a->gain_list[i] = gain;

}

/*=======================================================================*
 *                            dr_set_ClockSource                         *
 *=======================================================================*/
/* Set clock source.                                                     */

PUBLIC
void dr_set_ClockSource (timing_source)
int timing_source;
{

   a->info.timing_source = (timing_source == INTERNAL_CLOCK) ? 'i' : 'e';

}

/*=======================================================================*
 *                          dr_set_DigitizationRate                      *
 *=======================================================================*/
/* Set the digitization rate.                                            */

PUBLIC
void dr_set_DigitizationRate (rate)
double rate;
{
   digitization_rate = rate;
}

/*=======================================================================*
 *                           dr_set_TriggerSource                        *
 *=======================================================================*/
/* Set trigger source.                                                   */

PUBLIC
void dr_set_TriggerSource (trigger_source)
int trigger_source;
{

   a->info.trigger_source = (trigger_source == INTERNAL_TRIGGER) ? 'i' : 'e';

}

/*=======================================================================*
 *                            dr_start_collection
 *=======================================================================*/
/* Start continuous data acquisition.                                    */

PUBLIC
void dr_start_collection ()
{

dr_start_acq();

}

/*=======================================================================*
 *                            dr_stop_collection
 *=======================================================================*/
/* Stop continuous data acquisition.  Reset the DIGIREC.                 */

PUBLIC
void dr_stop_collection ()
{
dr_stop_acq();
}

/*=======================================================================*
 *                            ScanCheck
 *=======================================================================*/
ScanCheck(i)
int i;
{
if (i<scan_count) return(i);
else return(64);
}

